@@ -3,7 +3,7 @@ from uuid import uuid4
 
 # from dotenv import load_dotenv
 from flask import request, jsonify, Blueprint
-from openai import AsyncAzureOpenAI
+from openai import AzureOpenAI
 from ..batch.utilities.chat_history.auth_utils import (
     get_authenticated_user_details,
     get_tenantid,
@@ -34,13 +34,13 @@ def init_database_client():
 def init_openai_client():
     try:
         if env_helper.is_auth_type_keys():
-            azure_openai_client = AsyncAzureOpenAI(
+            azure_openai_client = AzureOpenAI(
                 azure_endpoint=env_helper.AZURE_OPENAI_ENDPOINT,
                 api_version=env_helper.AZURE_OPENAI_API_VERSION,
                 api_key=env_helper.AZURE_OPENAI_API_KEY,
             )
         else:
-            azure_openai_client = AsyncAzureOpenAI(
+            azure_openai_client = AzureOpenAI(
                 azure_endpoint=env_helper.AZURE_OPENAI_ENDPOINT,
                 api_version=env_helper.AZURE_OPENAI_API_VERSION,
                 azure_ad_token_provider=env_helper.AZURE_TOKEN_PROVIDER,
@@ -53,7 +53,7 @@ def init_openai_client():
 
 @bp_chat_history_response.route("/history/list", methods=["GET"])
 @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-async def list_conversations():
+def list_conversations():
     config = ConfigHelper.get_active_config_or_default()
     if not config.enable_chat_history:
         return jsonify({"error": "Chat history is not available"}), 400
@@ -71,9 +71,9 @@ async def list_conversations():
         if not conversation_client:
             return jsonify({"error": "Database not available"}), 500
 
-        await conversation_client.connect()
+        conversation_client.connect()
         try:
-            conversations = await conversation_client.get_conversations(
+            conversations = conversation_client.get_conversations(
                 user_id, offset=offset, limit=25, tenant_id=tenant_id
             )
             if not isinstance(conversations, list):
@@ -87,7 +87,7 @@ async def list_conversations():
             logger.exception(f"Error fetching conversations: {e}")
             raise
         finally:
-            await conversation_client.close()
+            conversation_client.close()
 
     except Exception as e:
         logger.exception(f"Exception in /history/list: {e}")
@@ -96,7 +96,7 @@ async def list_conversations():
 
 @bp_chat_history_response.route("/history/rename", methods=["POST"])
 @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-async def rename_conversation():
+def rename_conversation():
     config = ConfigHelper.get_active_config_or_default()
     if not config.enable_chat_history:
         return jsonify({"error": "Chat history is not available"}), 400
@@ -125,10 +125,10 @@ async def rename_conversation():
         if not conversation_client:
             return jsonify({"error": "Database not available"}), 500
 
-        await conversation_client.connect()
+        conversation_client.connect()
         try:
             # Retrieve conversation from database
-            conversation = await conversation_client.get_conversation(
+            conversation = conversation_client.get_conversation(
                 user_id, conversation_id, tenant_id=tenant_id
             )
             if not conversation:
@@ -143,7 +143,7 @@ async def rename_conversation():
 
             # Update the title and save changes
             conversation["title"] = title
-            updated_conversation = await conversation_client.upsert_conversation(
+            updated_conversation = conversation_client.upsert_conversation(
                 conversation, tenant_id=tenant_id
             )
             return jsonify(updated_conversation), 200
@@ -153,7 +153,7 @@ async def rename_conversation():
             )
             raise
         finally:
-            await conversation_client.close()
+            conversation_client.close()
     except Exception as e:
         logger.exception(f"Exception in /history/rename: {e}")
         return jsonify({"error": "Error while renaming conversation"}), 500
@@ -161,7 +161,7 @@ async def rename_conversation():
 
 @bp_chat_history_response.route("/history/read", methods=["POST"])
 @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-async def get_conversation():
+def get_conversation():
     config = ConfigHelper.get_active_config_or_default()
     if not config.enable_chat_history:
         return jsonify({"error": "Chat history is not available"}), 400
@@ -185,10 +185,10 @@ async def get_conversation():
         if not conversation_client:
             return jsonify({"error": "Database not available"}), 500
 
-        await conversation_client.connect()
+        conversation_client.connect()
         try:
             # Retrieve conversation
-            conversation = await conversation_client.get_conversation(
+            conversation = conversation_client.get_conversation(
                 user_id, conversation_id, tenant_id=tenant_id
             )
             if not conversation:
@@ -202,7 +202,7 @@ async def get_conversation():
                 )
 
             # Fetch conversation messages
-            conversation_messages = await conversation_client.get_messages(
+            conversation_messages = conversation_client.get_messages(
                 user_id, conversation_id, tenant_id=tenant_id
             )
             messages = [
@@ -227,7 +227,7 @@ async def get_conversation():
             )
             raise
         finally:
-            await conversation_client.close()
+            conversation_client.close()
 
     except Exception as e:
         logger.exception(f"Exception in /history/read: {e}")
@@ -236,7 +236,7 @@ async def get_conversation():
 
 @bp_chat_history_response.route("/history/delete", methods=["DELETE"])
 @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-async def delete_conversation():
+def delete_conversation():
     config = ConfigHelper.get_active_config_or_default()
     if not config.enable_chat_history:
         return jsonify({"error": "Chat history is not available"}), 400
@@ -268,39 +268,33 @@ async def delete_conversation():
         if not conversation_client:
             return jsonify({"error": "Database not available"}), 500
 
-        await conversation_client.connect()
+        conversation_client.connect()
         try:
-            # Delete conversation messages from database
-            await conversation_client.delete_messages(conversation_id, user_id, tenant_id=tenant_id)
-
-            # Delete the conversation itself
-            await conversation_client.delete_conversation(user_id, conversation_id, tenant_id=tenant_id)
-
-            return (
-                jsonify(
-                    {
-                        "message": "Successfully deleted conversation and messages",
-                        "conversation_id": conversation_id,
-                    }
-                ),
-                200,
+            # Delete messages and conversation
+            conversation_client.delete_messages(
+                user_id=user_id, conversation_id=conversation_id, tenant_id=tenant_id
             )
+            conversation_client.delete_conversation(
+                user_id=user_id, conversation_id=conversation_id, tenant_id=tenant_id
+            )
+
+            return jsonify({}), 204
         except Exception as e:
             logger.exception(
                 f"Error deleting conversation: user_id={user_id}, conversation_id={conversation_id}, error={e}"
             )
             raise
         finally:
-            await conversation_client.close()
+            conversation_client.close()
 
     except Exception as e:
         logger.exception(f"Exception in /history/delete: {e}")
-        return jsonify({"error": "Error while deleting conversation history"}), 500
+        return jsonify({"error": "Error while deleting conversation"}), 500
 
 
 @bp_chat_history_response.route("/history/delete_all", methods=["DELETE"])
 @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-async def delete_all_conversations():
+def delete_all_conversations():
     config = ConfigHelper.get_active_config_or_default()
 
     # Check if chat history is available
@@ -321,61 +315,42 @@ async def delete_all_conversations():
         if not conversation_client:
             return jsonify({"error": "Database not available"}), 500
 
-        await conversation_client.connect()
+        conversation_client.connect()
         try:
-            # Get all conversations for the user
-            conversations = await conversation_client.get_conversations(
-                user_id, offset=0, limit=None, tenant_id=tenant_id
+            conversations = conversation_client.get_conversations(
+                user_id, tenant_id=tenant_id
             )
-            if not conversations:
-                return (
-                    jsonify({"error": f"No conversations found for user {user_id}"}),
-                    400,
-                )
-
-            # Delete each conversation and its associated messages
-            for conversation in conversations:
-                try:
-                    # Delete messages associated with the conversation
-                    await conversation_client.delete_messages(
-                        conversation["id"], user_id, tenant_id=tenant_id
+            if conversations:
+                for conversation in conversations:
+                    conversation_client.delete_messages(
+                        user_id=user_id,
+                        conversation_id=conversation["conversation_id"],
+                        tenant_id=tenant_id,
+                    )
+                    conversation_client.delete_conversation(
+                        user_id=user_id,
+                        conversation_id=conversation["conversation_id"],
+                        tenant_id=tenant_id,
                     )
 
-                    # Delete the conversation itself
-                    await conversation_client.delete_conversation(
-                        user_id, conversation["id"], tenant_id=tenant_id
-                    )
-
-                except Exception as e:
-                    # Log and continue with the next conversation if one fails
-                    logger.exception(
-                        f"Error deleting conversation {conversation['id']} for user {user_id}: {e}"
-                    )
-                    continue
-            return (
-                jsonify(
-                    {
-                        "message": f"Successfully deleted all conversations and messages for user {user_id}"
-                    }
-                ),
-                200,
-            )
+            return jsonify({}), 204
         except Exception as e:
-            logger.exception(
-                f"Error deleting all conversations for user {user_id}: {e}"
-            )
+            logger.exception(f"Error deleting all conversations: {e}")
             raise
         finally:
-            await conversation_client.close()
+            conversation_client.close()
 
     except Exception as e:
         logger.exception(f"Exception in /history/delete_all: {e}")
-        return jsonify({"error": "Error while deleting all conversation history"}), 500
+        return (
+            jsonify({"error": "Error while deleting all historical conversations"}),
+            500,
+        )
 
 
 @bp_chat_history_response.route("/history/update", methods=["POST"])
 @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-async def update_conversation():
+def update_conversation():
     config = ConfigHelper.get_active_config_or_default()
     if not config.enable_chat_history:
         return jsonify({"error": "Chat history is not available"}), 400
@@ -394,91 +369,54 @@ async def update_conversation():
         if not conversation_id:
             return jsonify({"error": "conversation_id is required"}), 400
 
-        messages = request_json["messages"]
-        if not messages or len(messages) == 0:
-            return jsonify({"error": "Messages are required"}), 400
-
         # Initialize conversation client
         conversation_client = init_database_client()
         if not conversation_client:
             return jsonify({"error": "Database not available"}), 500
-        await conversation_client.connect()
+
+        conversation_client.connect()
         try:
-            # Get or create the conversation
-            conversation = await conversation_client.get_conversation(
+            # Create a new message in the conversation
+            new_message = {"role": "user", "content": request_json["messages"][0]["content"]}
+            message_id = str(uuid4())
+            message = conversation_client.create_message(
+                uuid=message_id,
+                conversation_id=conversation_id,
+                user_id=user_id,
+                input_message=new_message,
+                tenant_id=tenant_id,
+            )
+
+            # Check if a title needs to be generated
+            conversation = conversation_client.get_conversation(
                 user_id, conversation_id, tenant_id=tenant_id
             )
-            if not conversation:
-                title = await generate_title(messages)
-                conversation = await conversation_client.create_conversation(
-                    user_id=user_id, conversation_id=conversation_id, title=title, tenant_id=tenant_id
+            if conversation and (
+                conversation.get("title") is None
+                or conversation.get("title") == "New chat"
+            ):
+                conversation_messages = conversation_client.get_messages(
+                    user_id, conversation_id, tenant_id=tenant_id
                 )
-
-            # Process and save user and assistant messages
-            # Process user message
-            if messages[0]["role"] == "user":
-                user_message = next(
-                    (msg for msg in reversed(messages) if msg["role"] == "user"), None
-                )
-                if not user_message:
-                    return jsonify({"error": "User message not found"}), 400
-
-                created_message = await conversation_client.create_message(
-                    uuid=str(uuid4()),
-                    conversation_id=conversation_id,
-                    user_id=user_id,
-                    input_message=user_message,
-                    tenant_id=tenant_id,
-                )
-                if created_message == "Conversation not found":
-                    return jsonify({"error": "Conversation not found"}), 400
-
-            # Process assistant and tool messages if available
-            if messages[-1]["role"] == "assistant":
-                if len(messages) > 1 and messages[-2].get("role") == "tool":
-                    # Write the tool message first if it exists
-                    await conversation_client.create_message(
-                        uuid=str(uuid4()),
-                        conversation_id=conversation_id,
-                        user_id=user_id,
-                        input_message=messages[-2],
-                        tenant_id=tenant_id,
+                title = generate_title(conversation_messages)
+                if title:
+                    conversation["title"] = title
+                    conversation_client.upsert_conversation(
+                        conversation, tenant_id=tenant_id
                     )
-                # Write the assistant message
-                await conversation_client.create_message(
-                    uuid=str(uuid4()),
-                    conversation_id=conversation_id,
-                    user_id=user_id,
-                    input_message=messages[-1],
-                    tenant_id=tenant_id,
-                )
-            else:
-                return jsonify({"error": "No assistant message found"}), 400
 
-            return (
-                jsonify(
-                    {
-                        "success": True,
-                        "data": {
-                            "title": conversation["title"],
-                            "date": conversation["updatedAt"],
-                            "conversation_id": conversation["id"],
-                        },
-                    }
-                ),
-                200,
-            )
+            return jsonify(message), 200
         except Exception as e:
             logger.exception(
-                f"Error updating conversation or messages: user_id={user_id}, conversation_id={conversation_id}, error={e}"
+                f"Error creating message: user_id={user_id}, conversation_id={conversation_id}, error={e}"
             )
             raise
         finally:
-            await conversation_client.close()
+            conversation_client.close()
 
     except Exception as e:
         logger.exception(f"Exception in /history/update: {e}")
-        return jsonify({"error": "Error while updating the conversation history"}), 500
+        return jsonify({"error": "Error while updating conversation"}), 500
 
 
 @bp_chat_history_response.route("/history/frontend_settings", methods=["GET"])
@@ -503,36 +441,33 @@ def get_frontend_settings():
         return jsonify({"error": "Error while getting frontend settings"}), 500
 
 
-async def generate_title(conversation_messages):
-    title_prompt = "Summarize the conversation so far into a 4-word or less title. Do not use any quotation marks or punctuation. Do not include any other commentary or description."
-
-    # Filter only the user messages, but consider including system or assistant context if necessary
-    messages = [
-        {"role": msg["role"], "content": msg["content"]}
-        for msg in conversation_messages
-        if msg["role"] == "user"
+def generate_title(conversation_messages):
+    """
+    Generates a title for a conversation based on the messages.
+    """
+    # Format messages for the OpenAI API
+    formatted_messages = [
+        {"role": "system", "content": "You are a helpful assistant."}
     ]
-    messages.append({"role": "user", "content": title_prompt})
+    for message in conversation_messages:
+        formatted_messages.append({"role": message["role"], "content": message["content"]})
 
     try:
         azure_openai_client = init_openai_client()
+        if not azure_openai_client:
+            raise Exception("Azure OpenAI client not available")
 
-        # Create a chat completion with the Azure OpenAI client
-        response = await azure_openai_client.chat.completions.create(
-            model=env_helper.AZURE_OPENAI_MODEL,
-            messages=messages,
-            temperature=1,
-            max_tokens=64,
+        # Use the first user message as the basis for the title
+        response = azure_openai_client.chat.completions.create(
+            model=env_helper.AZURE_OPENAI_GPT_DEPLOYMENT,
+            messages=formatted_messages,
+            max_tokens=50,
+            temperature=0,
         )
 
-        # Ensure response contains valid choices and content
-        if response and response.choices and len(response.choices) > 0:
-            title = response.choices[0].message.content.strip()
-            return title
-        else:
-            raise ValueError("No valid choices in response")
+        title = response.choices[0].message.content
+        return title
 
     except Exception as e:
-        logger.exception(f"Error generating title: {str(e)}")
-        # Fallback: return the content of the second to last message if something goes wrong
-        return messages[-2]["content"] if len(messages) > 1 else "Untitled"
+        logger.exception(f"Error generating title: {e}")
+        return None
