@@ -1,4 +1,5 @@
-import asyncpg
+import psycopg
+from psycopg.rows import dict_row
 from datetime import datetime, timezone
 from .database_client_base import DatabaseClientBase
 from ...utilities.helpers.env_helper import EnvHelper
@@ -21,17 +22,18 @@ class PostgresConversationClient(DatabaseClientBase):
         self.conn = None
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def connect(self):
+    def connect(self):
         try:
             access_information = identity.get_token(scopes="https://ossrdbms-aad.database.windows.net/.default")
             token = access_information.token
-            self.conn = await asyncpg.connect(
+            self.conn = psycopg.connect(
                 user=self.user,
                 host=self.host,
-                database=self.database,
+                dbname=self.database,
                 password=token,
                 port=5432,
-                ssl="require",
+                sslmode="require",
+                row_factory=dict_row
             )
             logger.info("Successfully connected to PostgreSQL")
         except Exception as e:
@@ -39,13 +41,13 @@ class PostgresConversationClient(DatabaseClientBase):
             raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def close(self):
+    def close(self):
         if self.conn:
-            await self.conn.close()
+            self.conn.close()
             logger.info("PostgreSQL connection closed")
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def ensure(self):
+    def ensure(self):
         if not self.conn:
             logger.warning("PostgreSQL client not initialized correctly")
             return False, "PostgreSQL client not initialized correctly"
@@ -53,21 +55,24 @@ class PostgresConversationClient(DatabaseClientBase):
         return True, "PostgreSQL client initialized successfully"
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def create_conversation(self, conversation_id, user_id, title="", tenant_id=None):
+    def create_conversation(self, conversation_id, user_id, title="", tenant_id=None):
         utc_now = datetime.now(timezone.utc)
         createdAt = utc_now.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"
         query = """
             INSERT INTO conversations (id, conversation_id, type, "createdAt", "updatedAt", user_id, title, tenant_id)
-            VALUES ($1, $2, 'conversation', $3, $3, $4, $5, $6)
+            VALUES (%s, %s, 'conversation', %s, %s, %s, %s, %s)
             RETURNING *
         """
         try:
-            conversation = await self.conn.fetchrow(
-                query, conversation_id, conversation_id, createdAt, user_id, title, tenant_id
-            )
+            with self.conn.cursor() as cur:
+                cur.execute(
+                    query, (conversation_id, conversation_id, createdAt, createdAt, user_id, title, tenant_id)
+                )
+                conversation = cur.fetchone()
+                self.conn.commit()
             if conversation:
                 logger.info(f"Conversation created with id: {conversation_id}")
-                return dict(conversation)
+                return conversation
             else:
                 logger.warning(
                     f"Failed to create conversation with id: {conversation_id}"
@@ -78,13 +83,14 @@ class PostgresConversationClient(DatabaseClientBase):
                 f"Error creating conversation with id: {conversation_id}: {e}",
                 exc_info=True,
             )
+            self.conn.rollback()
             raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def upsert_conversation(self, conversation, tenant_id=None):
+    def upsert_conversation(self, conversation, tenant_id=None):
         query = """
             INSERT INTO conversations (id, conversation_id, type, "createdAt", "updatedAt", user_id, title, tenant_id)
-            VALUES ($1, $2, $3, $4, $5, $6, $7, $8)
+            VALUES (%s, %s, %s, %s, %s, %s, %s, %s)
             ON CONFLICT (id) DO UPDATE SET
                 "updatedAt" = EXCLUDED."updatedAt",
                 title = EXCLUDED.title,
@@ -92,20 +98,25 @@ class PostgresConversationClient(DatabaseClientBase):
             RETURNING *
         """
         try:
-            updated_conversation = await self.conn.fetchrow(
-                query,
-                conversation["id"],
-                conversation["conversation_id"],
-                conversation["type"],
-                conversation["createdAt"],
-                conversation["updatedAt"],
-                conversation["user_id"],
-                conversation["title"],
-                tenant_id,
-            )
+            with self.conn.cursor() as cur:
+                cur.execute(
+                    query,
+                    (
+                        conversation["id"],
+                        conversation["conversation_id"],
+                        conversation["type"],
+                        conversation["createdAt"],
+                        conversation["updatedAt"],
+                        conversation["user_id"],
+                        conversation["title"],
+                        tenant_id,
+                    ),
+                )
+                updated_conversation = cur.fetchone()
+                self.conn.commit()
             if updated_conversation:
                 logger.info(f"Conversation upserted with id: {conversation['id']}")
-                return dict(updated_conversation)
+                return updated_conversation
             else:
                 logger.warning(
                     f"Failed to upsert conversation with id: {conversation['id']}"
@@ -116,16 +127,19 @@ class PostgresConversationClient(DatabaseClientBase):
                 f"Error upserting conversation with id: {conversation['id']}: {e}",
                 exc_info=True,
             )
+            self.conn.rollback()
             raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def delete_conversation(self, user_id, conversation_id, tenant_id=None):
+    def delete_conversation(self, user_id, conversation_id, tenant_id=None):
         if tenant_id:
             query = (
-                "DELETE FROM conversations WHERE conversation_id = $1 AND user_id = $2 AND tenant_id = $3"
+                "DELETE FROM conversations WHERE conversation_id = %s AND user_id = %s AND tenant_id = %s"
             )
             try:
-                await self.conn.execute(query, conversation_id, user_id, tenant_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (conversation_id, user_id, tenant_id))
+                    self.conn.commit()
                 logger.info(
                     f"Conversation deleted with conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                 )
@@ -135,13 +149,16 @@ class PostgresConversationClient(DatabaseClientBase):
                     f"Error deleting conversation with conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
                     exc_info=True,
                 )
+                self.conn.rollback()
                 raise
         else:
             query = (
-                "DELETE FROM conversations WHERE conversation_id = $1 AND user_id = $2"
+                "DELETE FROM conversations WHERE conversation_id = %s AND user_id = %s"
             )
             try:
-                await self.conn.execute(query, conversation_id, user_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (conversation_id, user_id))
+                    self.conn.commit()
                 logger.info(
                     f"Conversation deleted with conversation_id: {conversation_id} and user_id: {user_id}"
                 )
@@ -151,61 +168,76 @@ class PostgresConversationClient(DatabaseClientBase):
                     f"Error deleting conversation with conversation_id: {conversation_id} and user_id: {user_id}: {e}",
                     exc_info=True,
                 )
+                self.conn.rollback()
                 raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def delete_messages(self, conversation_id, user_id, tenant_id=None):
+    def delete_messages(self, conversation_id, user_id, tenant_id=None):
         if tenant_id:
-            query = "DELETE FROM messages WHERE conversation_id = $1 AND user_id = $2 AND tenant_id = $3 RETURNING *"
+            query = "DELETE FROM messages WHERE conversation_id = %s AND user_id = %s AND tenant_id = %s RETURNING *"
             try:
-                messages = await self.conn.fetch(query, conversation_id, user_id, tenant_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (conversation_id, user_id, tenant_id))
+                    messages = cur.fetchall()
+                    self.conn.commit()
                 logger.info(
                     f"Messages deleted for conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                 )
-                return [dict(message) for message in messages]
+                return messages
             except Exception as e:
                 logger.error(
                     f"Error deleting messages for conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
                     exc_info=True,
                 )
+                self.conn.rollback()
                 raise
         else:
-            query = "DELETE FROM messages WHERE conversation_id = $1 AND user_id = $2 RETURNING *"
+            query = "DELETE FROM messages WHERE conversation_id = %s AND user_id = %s RETURNING *"
             try:
-                messages = await self.conn.fetch(query, conversation_id, user_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (conversation_id, user_id))
+                    messages = cur.fetchall()
+                    self.conn.commit()
                 logger.info(
                     f"Messages deleted for conversation_id: {conversation_id} and user_id: {user_id}"
                 )
-                return [dict(message) for message in messages]
+                return messages
             except Exception as e:
                 logger.error(
                     f"Error deleting messages for conversation_id: {conversation_id} and user_id: {user_id}: {e}",
                     exc_info=True,
                 )
+                self.conn.rollback()
                 raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def get_conversations(self, user_id, limit=None, sort_order="DESC", offset=0, tenant_id=None):
+    def get_conversations(self, user_id, limit=None, sort_order="DESC", offset=0, tenant_id=None):
         try:
             offset = int(offset)  # Ensure offset is an integer
         except ValueError:
             logger.error("Offset must be an integer.", exc_info=True)
             raise ValueError("Offset must be an integer.")
         
+        params = [user_id]
+        
         if tenant_id:
             # Base query without LIMIT and OFFSET
             query = f"""
                 SELECT * FROM conversations
-                WHERE user_id = $1 AND type = 'conversation' AND tenant_id = $2
+                WHERE user_id = %s AND type = 'conversation' AND tenant_id = %s
                 ORDER BY "updatedAt" {sort_order}
             """
+            params.append(tenant_id)
             # Append LIMIT and OFFSET to the query if limit is specified
             if limit is not None:
                 try:
                     limit = int(limit)  # Ensure limit is an integer
-                    query += " LIMIT $3 OFFSET $4"
+                    query += " LIMIT %s OFFSET %s"
+                    params.extend([limit, offset])
                     # Fetch records with LIMIT and OFFSET
-                    conversations = await self.conn.fetch(query, user_id, tenant_id, limit, offset)
+                    with self.conn.cursor() as cur:
+                        cur.execute(query, tuple(params))
+                        conversations = cur.fetchall()
                     logger.info(
                         f"Retrieved conversations for user_id: {user_id}, tenant_id: {tenant_id} with limit: {limit} and offset: {offset}"
                     )
@@ -214,7 +246,9 @@ class PostgresConversationClient(DatabaseClientBase):
                     raise ValueError("Limit must be an integer.")
             else:
                 # Fetch records without LIMIT and OFFSET
-                conversations = await self.conn.fetch(query, user_id, tenant_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, tuple(params))
+                    conversations = cur.fetchall()
                 logger.info(
                     f"Retrieved conversations for user_id: {user_id}, tenant_id: {tenant_id} without limit and offset"
                 )
@@ -222,16 +256,19 @@ class PostgresConversationClient(DatabaseClientBase):
             # Base query without LIMIT and OFFSET
             query = f"""
                 SELECT * FROM conversations
-                WHERE user_id = $1 AND type = 'conversation'
+                WHERE user_id = %s AND type = 'conversation'
                 ORDER BY "updatedAt" {sort_order}
             """
             # Append LIMIT and OFFSET to the query if limit is specified
             if limit is not None:
                 try:
                     limit = int(limit)  # Ensure limit is an integer
-                    query += " LIMIT $2 OFFSET $3"
+                    query += " LIMIT %s OFFSET %s"
+                    params.extend([limit, offset])
                     # Fetch records with LIMIT and OFFSET
-                    conversations = await self.conn.fetch(query, user_id, limit, offset)
+                    with self.conn.cursor() as cur:
+                        cur.execute(query, tuple(params))
+                        conversations = cur.fetchall()
                     logger.info(
                         f"Retrieved conversations for user_id: {user_id} with limit: {limit} and offset: {offset}"
                     )
@@ -240,211 +277,207 @@ class PostgresConversationClient(DatabaseClientBase):
                     raise ValueError("Limit must be an integer.")
             else:
                 # Fetch records without LIMIT and OFFSET
-                conversations = await self.conn.fetch(query, user_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, tuple(params))
+                    conversations = cur.fetchall()
                 logger.info(
                     f"Retrieved conversations for user_id: {user_id} without limit and offset"
                 )
-        return [dict(conversation) for conversation in conversations]
+        return conversations
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def get_conversation(self, user_id, conversation_id, tenant_id=None):
+    def get_conversation(self, user_id, conversation_id, tenant_id=None):
         if tenant_id:
-            query = "SELECT * FROM conversations WHERE id = $1 AND user_id = $2 AND type = 'conversation' AND tenant_id = $3"
+            query = "SELECT * FROM conversations WHERE user_id = %s AND conversation_id = %s AND type = 'conversation' AND tenant_id = %s"
             try:
-                conversation = await self.conn.fetchrow(query, conversation_id, user_id, tenant_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (user_id, conversation_id, tenant_id))
+                    conversation = cur.fetchone()
                 if conversation:
                     logger.info(
-                        f"Retrieved conversation with id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
+                        f"Conversation retrieved with conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                     )
-                    return dict(conversation)
+                    return conversation
                 else:
                     logger.warning(
-                        f"No conversation found with id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
+                        f"Conversation not found with conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                     )
                     return None
             except Exception as e:
                 logger.error(
-                    f"Error retrieving conversation with id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
+                    f"Error getting conversation with conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
                     exc_info=True,
                 )
                 raise
         else:
-            query = "SELECT * FROM conversations WHERE id = $1 AND user_id = $2 AND type = 'conversation'"
+            query = "SELECT * FROM conversations WHERE user_id = %s AND conversation_id = %s AND type = 'conversation'"
             try:
-                conversation = await self.conn.fetchrow(query, conversation_id, user_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (user_id, conversation_id))
+                    conversation = cur.fetchone()
                 if conversation:
                     logger.info(
-                        f"Retrieved conversation with id: {conversation_id} and user_id: {user_id}"
+                        f"Conversation retrieved with conversation_id: {conversation_id} and user_id: {user_id}"
                     )
-                    return dict(conversation)
+                    return conversation
                 else:
                     logger.warning(
-                        f"No conversation found with id: {conversation_id} and user_id: {user_id}"
+                        f"Conversation not found with conversation_id: {conversation_id} and user_id: {user_id}"
                     )
                     return None
             except Exception as e:
                 logger.error(
-                    f"Error retrieving conversation with id: {conversation_id} and user_id: {user_id}: {e}",
+                    f"Error getting conversation with conversation_id: {conversation_id} and user_id: {user_id}: {e}",
                     exc_info=True,
                 )
                 raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def create_message(self, uuid, conversation_id, user_id, input_message: dict, tenant_id=None):
-        message_id = uuid
+    def create_message(self, uuid, conversation_id, user_id, input_message: dict, tenant_id=None):
         utc_now = datetime.now(timezone.utc)
         createdAt = utc_now.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"
-        
+
+        message = {
+            "id": uuid,
+            "conversation_id": conversation_id,
+            "createdAt": createdAt,
+            "role": input_message.get("role", "user"),
+            "user_id": user_id,
+            "content": input_message["content"],
+            "tenant_id": tenant_id,
+            "type": "message",
+            "feedback": input_message.get("feedback", "none"),
+            "metadata": input_message.get("metadata", {}),
+        }
+
+        insert_query = """
+            INSERT INTO messages (id, conversation_id, "createdAt", role, user_id, content, tenant_id, type, feedback, metadata)
+            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
+            RETURNING *
+        """
+        update_query = 'UPDATE conversations SET "updatedAt" = %s WHERE id = %s AND user_id = %s'
         if tenant_id:
-            query = """
-                INSERT INTO messages (id, type, "createdAt", "updatedAt", user_id, conversation_id, role, content, feedback, tenant_id)
-                VALUES ($1, 'message', $2, $2, $3, $4, $5, $6, $7, $8)
-                RETURNING *
-            """
-            feedback = "" if self.enable_message_feedback else None
-            try:
-                message = await self.conn.fetchrow(
-                    query,
-                    message_id,
-                    createdAt,
-                    user_id,
-                    conversation_id,
-                    input_message["role"],
-                    input_message["content"],
-                    feedback,
-                    tenant_id,
-                )
+            update_query += " AND tenant_id = %s"
 
-                if message:
-                    update_query = 'UPDATE conversations SET "updatedAt" = $1 WHERE id = $2 AND user_id = $3 AND tenant_id = $4 RETURNING *'
-                    await self.conn.execute(
-                        update_query, createdAt, conversation_id, user_id, tenant_id
-                    )
-                    logger.info(
-                        f"Message created with id: {message_id} in conversation: {conversation_id}, tenant_id: {tenant_id}"
-                    )
-                    return dict(message)
-                else:
-                    logger.warning(
-                        f"Failed to create message with id: {message_id} in conversation: {conversation_id}, tenant_id: {tenant_id}"
-                    )
-                    return False
-            except Exception as e:
-                logger.error(
-                    f"Error creating message with id: {message_id} in conversation: {conversation_id}, tenant_id: {tenant_id}: {e}",
-                    exc_info=True,
-                )
-                raise
-        else:
-            query = """
-                INSERT INTO messages (id, type, "createdAt", "updatedAt", user_id, conversation_id, role, content, feedback)
-                VALUES ($1, 'message', $2, $2, $3, $4, $5, $6, $7)
-                RETURNING *
-            """
-            feedback = "" if self.enable_message_feedback else None
-            try:
-                message = await self.conn.fetchrow(
-                    query,
-                    message_id,
-                    createdAt,
-                    user_id,
-                    conversation_id,
-                    input_message["role"],
-                    input_message["content"],
-                    feedback,
+        try:
+            with self.conn.cursor() as cur:
+                cur.execute(
+                    insert_query,
+                    (
+                        message["id"],
+                        message["conversation_id"],
+                        message["createdAt"],
+                        message["role"],
+                        message["user_id"],
+                        message["content"],
+                        message["tenant_id"],
+                        message["type"],
+                        message["feedback"],
+                        message["metadata"],
+                    ),
                 )
+                created_message = cur.fetchone()
 
-                if message:
-                    update_query = 'UPDATE conversations SET "updatedAt" = $1 WHERE id = $2 AND user_id = $3 RETURNING *'
-                    await self.conn.execute(
-                        update_query, createdAt, conversation_id, user_id
-                    )
-                    logger.info(
-                        f"Message created with id: {message_id} in conversation: {conversation_id}"
-                    )
-                    return dict(message)
-                else:
-                    logger.warning(
-                        f"Failed to create message with id: {message_id} in conversation: {conversation_id}"
-                    )
-                    return False
-            except Exception as e:
-                logger.error(
-                    f"Error creating message with id: {message_id} in conversation: {conversation_id}: {e}",
-                    exc_info=True,
-                )
-                raise
+                update_params = (createdAt, conversation_id, user_id)
+                if tenant_id:
+                    update_params += (tenant_id,)
+                cur.execute(update_query, update_params)
+
+                self.conn.commit()
+
+            if created_message:
+                logger.info(f"Message created with id: {uuid}")
+                return created_message
+            else:
+                logger.warning(f"Failed to create message with id: {uuid}")
+                return False
+        except Exception as e:
+            logger.error(f"Error creating message with id: {uuid}: {e}", exc_info=True)
+            self.conn.rollback()
+            raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def update_message_feedback(self, user_id, message_id, feedback, tenant_id=None):
+    def update_message_feedback(self, user_id, message_id, feedback, tenant_id=None):
         if tenant_id:
-            query = "UPDATE messages SET feedback = $1 WHERE id = $2 AND user_id = $3 AND tenant_id = $4 RETURNING *"
+            query = "UPDATE messages SET feedback = %s WHERE id = %s AND user_id = %s AND tenant_id = %s RETURNING *"
             try:
-                message = await self.conn.fetchrow(query, feedback, message_id, user_id, tenant_id)
-                if message:
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (feedback, message_id, user_id, tenant_id))
+                    updated_message = cur.fetchone()
+                    self.conn.commit()
+                if updated_message:
                     logger.info(
                         f"Message feedback updated for message_id: {message_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                     )
-                    return dict(message)
+                    return updated_message
                 else:
                     logger.warning(
-                        f"Failed to update message feedback for message_id: {message_id}, user_id: {user_id}, tenant_id: {tenant_id}"
+                        f"Message not found or feedback not updated for message_id: {message_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                     )
-                    return False
+                    return None
             except Exception as e:
                 logger.error(
                     f"Error updating message feedback for message_id: {message_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
                     exc_info=True,
                 )
+                self.conn.rollback()
                 raise
         else:
-            query = "UPDATE messages SET feedback = $1 WHERE id = $2 AND user_id = $3 RETURNING *"
+            query = "UPDATE messages SET feedback = %s WHERE id = %s AND user_id = %s RETURNING *"
             try:
-                message = await self.conn.fetchrow(query, feedback, message_id, user_id)
-                if message:
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (feedback, message_id, user_id))
+                    updated_message = cur.fetchone()
+                    self.conn.commit()
+                if updated_message:
                     logger.info(
                         f"Message feedback updated for message_id: {message_id} and user_id: {user_id}"
                     )
-                    return dict(message)
+                    return updated_message
                 else:
                     logger.warning(
-                        f"Failed to update message feedback for message_id: {message_id} and user_id: {user_id}"
+                        f"Message not found or feedback not updated for message_id: {message_id} and user_id: {user_id}"
                     )
-                    return False
+                    return None
             except Exception as e:
                 logger.error(
                     f"Error updating message feedback for message_id: {message_id} and user_id: {user_id}: {e}",
                     exc_info=True,
                 )
+                self.conn.rollback()
                 raise
 
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=log_result)
-    async def get_messages(self, user_id, conversation_id, tenant_id=None):
+    def get_messages(self, user_id, conversation_id, tenant_id=None):
         if tenant_id:
-            query = 'SELECT * FROM messages WHERE conversation_id = $1 AND user_id = $2 AND tenant_id = $3 ORDER BY "createdAt" ASC'
+            query = "SELECT * FROM messages WHERE conversation_id = %s AND user_id = %s AND tenant_id = %s ORDER BY \"createdAt\" ASC"
             try:
-                messages = await self.conn.fetch(query, conversation_id, user_id, tenant_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (conversation_id, user_id, tenant_id))
+                    messages = cur.fetchall()
                 logger.info(
-                    f"Retrieved messages for conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
+                    f"Messages retrieved for conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}"
                 )
-                return [dict(message) for message in messages]
+                return messages
             except Exception as e:
                 logger.error(
-                    f"Error retrieving messages for conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
+                    f"Error getting messages for conversation_id: {conversation_id}, user_id: {user_id}, tenant_id: {tenant_id}: {e}",
                     exc_info=True,
                 )
                 raise
         else:
-            query = 'SELECT * FROM messages WHERE conversation_id = $1 AND user_id = $2 ORDER BY "createdAt" ASC'
+            query = "SELECT * FROM messages WHERE conversation_id = %s AND user_id = %s ORDER BY \"createdAt\" ASC"
             try:
-                messages = await self.conn.fetch(query, conversation_id, user_id)
+                with self.conn.cursor() as cur:
+                    cur.execute(query, (conversation_id, user_id))
+                    messages = cur.fetchall()
                 logger.info(
-                    f"Retrieved messages for conversation_id: {conversation_id} and user_id: {user_id}"
+                    f"Messages retrieved for conversation_id: {conversation_id} and user_id: {user_id}"
                 )
-                return [dict(message) for message in messages]
+                return messages
             except Exception as e:
                 logger.error(
-                    f"Error retrieving messages for conversation_id: {conversation_id} and user_id: {user_id}: {e}",
+                    f"Error getting messages for conversation_id: {conversation_id} and user_id: {user_id}: {e}",
                     exc_info=True,
                 )
                 raise
