@@ -15,7 +15,7 @@ class Orchestrator:
         self.orchestrator = SemanticKernelOrchestrator()
 
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=False)
-    async def handle_message(
+    def handle_message(
         self,
         user_message: str,
         chat_history: List[dict],
@@ -24,6 +24,6 @@ class Orchestrator:
         tenant_id: str = None,
         **kwargs: dict,
     ) -> dict:
-        return await self.orchestrator.handle_message(
+        return self.orchestrator.handle_message(
             user_message, chat_history, conversation_id, user_info, tenant_id, **kwargs
         )
