@@ -36,8 +36,9 @@ class AzureSearchIndex:
         self.env_helper = env_helper
         self.llm_helper = llm_helper
 
+        validated_endpoint = self._validate_search_endpoint(self.env_helper.AZURE_SEARCH_SERVICE)
         self.index_client = SearchIndexClient(
-            self.env_helper.AZURE_SEARCH_SERVICE,
+            validated_endpoint,
             (
                 AzureKeyCredential(self.env_helper.AZURE_SEARCH_KEY)
                 if self.env_helper.is_auth_type_keys()
@@ -45,6 +46,44 @@ class AzureSearchIndex:
             ),
         )
 
+    def _validate_search_endpoint(self, endpoint: str) -> str:
+        """
+        Validate and normalize the Azure Search endpoint URL.
+        
+        Args:
+            endpoint: The endpoint URL to validate
+            
+        Returns:
+            Normalized HTTPS endpoint URL
+            
+        Raises:
+            ValueError: If the endpoint is invalid or not HTTPS when required
+        """
+        if not endpoint:
+            raise ValueError("Azure Search endpoint URL is required but not provided")
+        
+        # Normalize the URL
+        endpoint = endpoint.strip()
+        
+        # Add https:// if no protocol is specified
+        if not endpoint.startswith(('http://', 'https://')):
+            endpoint = f"https://{endpoint}"
+        
+        # Ensure HTTPS when using Bearer token authentication
+        if not self.env_helper.is_auth_type_keys():
+            if not endpoint.startswith('https://'):
+                raise ValueError(
+                    f"Azure Search endpoint must use HTTPS when using Bearer token authentication. "
+                    f"Current endpoint: {endpoint}"
+                )
+        
+        # Remove trailing slash if present
+        if endpoint.endswith('/'):
+            endpoint = endpoint[:-1]
+        
+        logger.info(f"Azure Search endpoint validated: {endpoint}")
+        return endpoint
+
     @property
     @logger.trace_function(log_execution=log_execution, log_args=log_args, log_result=False)
     def search_dimensions(self) -> int:
