@@ -1,3 +1,4 @@
+import asyncio
 import json
 from uuid import uuid4
 from typing import List, Optional
@@ -96,7 +97,7 @@ class SemanticKernelOrchestrator:
         return None
 
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=False)
-    async def orchestrate(
+    def orchestrate(
         self, user_message: str, chat_history: list[dict], user_info, tenant_id: str, config, **kwargs: dict
     ) -> list[dict]:
         """
@@ -175,13 +176,12 @@ class SemanticKernelOrchestrator:
         for message in chat_history.copy():
             history.add_message(message)
 
-        result: ChatMessageContent = (
-            await self.kernel.invoke(
-                function=orchestrate_function,
-                chat_history=history,
-                user_message=user_message,
-            )
-        ).value[0]
+        result_coro = self.kernel.invoke(
+            function=orchestrate_function,
+            chat_history=history,
+            user_message=user_message,
+        )
+        result: ChatMessageContent = asyncio.run(result_coro).value[0]
 
         self.log_tokens(
             prompt_tokens=result.metadata["usage"].prompt_tokens,
@@ -200,9 +200,8 @@ class SemanticKernelOrchestrator:
 
             arguments = json.loads(result.items[0].arguments)
 
-            answer: Answer = (
-                await self.kernel.invoke(function=function, **arguments)
-            ).value
+            answer_coro = self.kernel.invoke(function=function, **arguments)
+            answer: Answer = asyncio.run(answer_coro).value
 
             self.log_tokens(
                 prompt_tokens=answer.prompt_tokens,
@@ -215,13 +214,12 @@ class SemanticKernelOrchestrator:
                 and "search_documents" in function_name
             ):
                 logger.debug("Running post answering prompt")
-                answer: Answer = (
-                    await self.kernel.invoke(
-                        function_name="validate_answer",
-                        plugin_name="PostAnswering",
-                        answer=answer,
-                    )
-                ).value
+                answer_coro = self.kernel.invoke(
+                    function_name="validate_answer",
+                    plugin_name="PostAnswering",
+                    answer=answer,
+                )
+                answer: Answer = asyncio.run(answer_coro).value
 
                 self.log_tokens(
                     prompt_tokens=answer.prompt_tokens,
@@ -253,7 +251,7 @@ class SemanticKernelOrchestrator:
         return messages
 
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=False)
-    async def handle_message(
+    def handle_message(
         self,
         user_message: str,
         chat_history: List[dict],
@@ -263,50 +261,81 @@ class SemanticKernelOrchestrator:
         **kwargs: Optional[dict],
     ) -> dict:
         """
-        Handles the user message by orchestrating the conversation, logging token usage,
-        and logging user interactions if configured.
+        Handles the user's message, orchestrates the conversation, and logs the interaction.
 
         Args:
             user_message (str): The message from the user.
-            chat_history (List[dict]): The history of the chat as a list of dictionaries.
-            conversation_id (Optional[str]): The unique identifier for the conversation.
-            user_info: User information and request headers.
+            chat_history (List[dict]): The history of the chat.
+            conversation_id (str): The ID of the conversation.
+            user_info: Information about the user.
             tenant_id (str): The ID of the tenant.
-            **kwargs (Optional[dict]): Additional keyword arguments.
+            kwargs (dict): Additional keyword arguments.
 
         Returns:
-            dict: The result of the orchestration as a dictionary.
+            dict: The response to be sent to the user.
         """
-        config = ConfigHelper.get_active_config_or_default(tenant_id=tenant_id)
-        if config.logging.log_user_interactions:
-            self.conversation_logger.log_user_message(
-                self.message_id, conversation_id, user_info, user_message
-            )
-        messages = await self.orchestrate(
-            user_message, chat_history, user_info, tenant_id, config, **kwargs
-        )
-        if config.logging.log_user_interactions:
-            self.conversation_logger.log_assistant_message(
-                self.message_id, conversation_id, user_info, messages
-            )
-        if str(config.logging.log_tokens).lower() == "true":
-            custom_dimensions = {
+        logger.info("Handling message from user.")
+        config = ConfigHelper.get_active_config_or_default()
+        
+        # Log conversation initiation
+        self.conversation_logger.log(
+            data={
                 "conversation_id": conversation_id,
+                "user_id": user_info["user_principal_id"] if user_info else "anonymous",
                 "message_id": self.message_id,
-                "prompt_tokens": self.tokens["prompt"],
-                "completion_tokens": self.tokens["completion"],
-                "total_tokens": self.tokens["total"],
+                "tenant_id": tenant_id,
+                "user_question": user_message,
+                "type": "user",
             }
-            logger.info("Token Consumption", extra=custom_dimensions)
-        if str(config.logging.log_user_interactions).lower() == "true":
-            self.conversation_logger.log(
-                messages=[
-                    {
-                        "role": "user",
-                        "content": user_message,
-                        "conversation_id": conversation_id,
-                    }
-                ]
-                + messages
+        )
+        
+        # Orchestrate the response
+        response = self.orchestrate(
+            user_message=user_message,
+            chat_history=chat_history,
+            user_info=user_info,
+            tenant_id=tenant_id,
+            config=config,
+            **kwargs,
+        )
+        
+        # Log conversation completion
+        self.conversation_logger.log(
+            data={
+                "conversation_id": conversation_id,
+                "user_id": user_info["user_principal_id"] if user_info else "anonymous",
+                "message_id": self.message_id,
+                "tenant_id": tenant_id,
+                "user_question": user_message,
+                "model_response": response,
+                "type": "assistant",
+            }
+        )
+        
+        return response
+    
+    def call_content_safety_output(
+        self, user_message: str, answer: str
+    ) -> Optional[list[dict]]:
+        """
+        Validates the answer for harmful content and replaces it if necessary.
+
+        Args:
+            user_message (str): The message from the user.
+            answer (str): The answer generated by the system.
+
+        Returns:
+            Optional[list[dict]]: Parsed messages if harmful content is detected, otherwise None.
+        """
+        logger.debug("Calling content safety with answer")
+        filtered_answer = (
+            self.content_safety_checker.validate_output_and_replace_if_harmful(answer)
+        )
+        if answer != filtered_answer:
+            logger.warning("Content safety detected harmful content in answer")
+            messages = self.output_parser.parse(
+                question=user_message, answer=filtered_answer
             )
-        return messages
+            return messages
+
+        return None
