@@ -221,13 +221,14 @@ class QuestionAnswerTool(AnsweringToolBase):
         return messages
 
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=False)
-    def answer_question(self, question: str, chat_history: list[dict], **kwargs):
+    def answer_question(self, question: str, chat_history: list[dict], tenant_id: str, **kwargs):
         """
         Answer the given question using the chat history and additional parameters.
 
         Args:
             question (str): The question to be answered.
             chat_history (list[dict]): The chat history.
+            tenant_id (str): The ID of the tenant.
             **kwargs: Additional parameters.
 
         Returns:
@@ -235,7 +236,7 @@ class QuestionAnswerTool(AnsweringToolBase):
         """
         logger.info("Answering question")
         source_documents = Search.get_source_documents(
-            self.search_handler, question
+            self.search_handler, question, tenant_id
         )
 
         if self.env_helper.USE_ADVANCED_IMAGE_PROCESSING:
